@@ -3,6 +3,7 @@ package hudson.model;
 import hudson.ExtensionList;
 import hudson.ExtensionPoint;
 import hudson.scm.SCMDescriptor;
+import jenkins.ExtensionFilter;
 import jenkins.model.Jenkins;
 
 import java.util.ArrayList;
@@ -13,6 +14,7 @@ import java.util.List;
  *
  * @author Kohsuke Kawaguchi
  * @since 1.393
+ * @see ExtensionFilter
  */
 public abstract class DescriptorVisibilityFilter implements ExtensionPoint {
 
