@@ -24,14 +24,21 @@
  */
 package hudson.model;
 
+import com.thoughtworks.xstream.converters.ConversionException;
+import com.thoughtworks.xstream.io.StreamException;
+import com.thoughtworks.xstream.io.xml.XppDriver;
 import hudson.DescriptorExtensionList;
 import hudson.Extension;
 import hudson.ExtensionPoint;
+import hudson.Functions;
 import hudson.Indenter;
 import hudson.Util;
+import hudson.XmlFile;
 import hudson.model.Descriptor.FormException;
 import hudson.model.Node.Mode;
+import hudson.model.labels.LabelAtom;
 import hudson.model.labels.LabelAtomPropertyDescriptor;
+import hudson.scm.ChangeLogSet;
 import hudson.scm.ChangeLogSet.Entry;
 import hudson.search.CollectionSearchIndex;
 import hudson.search.SearchIndexBuilder;
@@ -40,22 +47,47 @@ import hudson.security.AccessControlled;
 import hudson.security.Permission;
 import hudson.security.PermissionGroup;
 import hudson.security.PermissionScope;
+import hudson.tasks.UserAvatarResolver;
 import hudson.util.AlternativeUiTextProvider;
 import hudson.util.AlternativeUiTextProvider.Message;
+import hudson.util.AtomicFileWriter;
 import hudson.util.DescribableList;
 import hudson.util.DescriptorList;
+import hudson.util.IOException2;
+import hudson.util.IOUtils;
 import hudson.util.RunList;
+import hudson.util.XStream2;
 import hudson.views.ListViewColumn;
 import hudson.widgets.Widget;
 import jenkins.model.Jenkins;
+import jenkins.util.ProgressiveRendering;
+import net.sf.json.JSON;
+import net.sf.json.JSONArray;
+import net.sf.json.JSONObject;
+import org.kohsuke.stapler.HttpResponse;
+import org.kohsuke.stapler.HttpResponses;
+import org.kohsuke.stapler.Stapler;
 import org.kohsuke.stapler.StaplerRequest;
 import org.kohsuke.stapler.StaplerResponse;
+import org.kohsuke.stapler.WebMethod;
 import org.kohsuke.stapler.export.Exported;
 import org.kohsuke.stapler.export.ExportedBean;
 import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import javax.servlet.ServletException;
+import javax.servlet.http.HttpServletResponse;
+import javax.xml.transform.Source;
+import javax.xml.transform.Transformer;
+import javax.xml.transform.TransformerException;
+import javax.xml.transform.TransformerFactory;
+import javax.xml.transform.stream.StreamResult;
+import javax.xml.transform.stream.StreamSource;
+import java.io.BufferedInputStream;
+import java.io.ByteArrayInputStream;
+import java.io.FileInputStream;
 import java.io.IOException;
+import java.io.InputStream;
+import java.io.StringWriter;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Calendar;
@@ -67,9 +99,11 @@ import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
+import static javax.servlet.http.HttpServletResponse.SC_BAD_REQUEST;
 import static jenkins.model.Jenkins.*;
 
 /**
@@ -383,7 +417,7 @@ public abstract class View extends AbstractModelObject implements AccessControll
         for (Computer c : computers) {
             Node n = c.getNode();
             if (n != null) {
-                if (labels.contains(null) && n.getMode() == Mode.NORMAL || !Collections.disjoint(n.getAssignedLabels(), labels)) {
+                if (labels.contains(null) && n.getMode() == Mode.NORMAL || !isDisjoint(n.getAssignedLabels(), labels)) {
                     result.add(c);
                 }
             }
@@ -391,7 +425,14 @@ public abstract class View extends AbstractModelObject implements AccessControll
 
         return result;
     }
-    
+
+    private boolean isDisjoint(Collection c1, Collection c2) {
+        for (Object o : c1)
+            if (c2.contains(o))
+                return false;
+        return true;
+    }
+
     public List<Queue.Item> getQueueItems() {
         if (!isFilterQueue()) {
             return Arrays.asList(Jenkins.getInstance().getQueue().getItems());
@@ -449,13 +490,17 @@ public abstract class View extends AbstractModelObject implements AccessControll
     	result.addAll(getOwnerViewActions());
     	synchronized (this) {
     		if (transientActions == null) {
-    			transientActions = TransientViewActionFactory.createAllFor(this); 
+                updateTransientActions();
     		}
     		result.addAll(transientActions);
     	}
     	return result;
     }
     
+    public synchronized void updateTransientActions() {
+        transientActions = TransientViewActionFactory.createAllFor(this); 
+    }
+    
     public Object getDynamic(String token) {
         for (Action a : getActions()) {
             String url = a.getUrlName();
@@ -594,6 +639,13 @@ public abstract class View extends AbstractModelObject implements AccessControll
         return new People(this);
     }
 
+    /**
+     * @since 1.484
+     */
+    public AsynchPeople getAsynchPeople() {
+        return new AsynchPeople(this);
+    }
+
     @ExportedBean
     public static final class People  {
         @Exported
@@ -675,6 +727,117 @@ public abstract class View extends AbstractModelObject implements AccessControll
         }
     }
 
+    /**
+     * Variant of {@link People} which can be displayed progressively, since it may be slow.
+     * @since 1.484
+     */
+    public static final class AsynchPeople extends ProgressiveRendering { // JENKINS-15206
+
+        private final Collection<TopLevelItem> items;
+        private final User unknown;
+        private final Map<User,UserInfo> users = new HashMap<User,UserInfo>();
+        private final Set<User> modified = new HashSet<User>();
+        private final String iconSize;
+        public final ModelObject parent;
+
+        /** @see Jenkins#getAsynchPeople} */
+        public AsynchPeople(Jenkins parent) {
+            this.parent = parent;
+            items = parent.getItems();
+            unknown = User.getUnknown();
+        }
+
+        /** @see View#getAsynchPeople */
+        public AsynchPeople(View parent) {
+            this.parent = parent;
+            items = parent.getItems();
+            unknown = null;
+        }
+
+        {
+            StaplerRequest req = Stapler.getCurrentRequest();
+            iconSize = req != null ? Functions.getCookie(req, "iconSize", "32x32") : "32x32";
+        }
+
+        @Override protected void compute() throws Exception {
+            int itemCount = 0;
+            for (Item item : items) {
+                for (Job<?,?> job : item.getAllJobs()) {
+                    if (job instanceof AbstractProject) {
+                        AbstractProject<?,?> p = (AbstractProject) job;
+                        RunList<? extends AbstractBuild<?,?>> builds = p.getBuilds();
+                        int buildCount = 0;
+                        for (AbstractBuild<?,?> build : builds) {
+                            if (canceled()) {
+                                return;
+                            }
+                            for (ChangeLogSet.Entry entry : build.getChangeSet()) {
+                                User user = entry.getAuthor();
+                                synchronized (this) {
+                                    UserInfo info = users.get(user);
+                                    if (info == null) {
+                                        users.put(user, new UserInfo(user, p, build.getTimestamp()));
+                                        modified.add(user);
+                                    } else if (info.getLastChange().before(build.getTimestamp())) {
+                                        info.project = p;
+                                        info.lastChange = build.getTimestamp();
+                                        modified.add(user);
+                                    }
+                                }
+                            }
+                            buildCount++;
+                            progress((itemCount + 1.0 * buildCount / builds.size()) / (items.size() + 1));
+                        }
+                    }
+                }
+                itemCount++;
+                progress(1.0 * itemCount / (items.size() + /* handling User.getAll */1));
+            }
+            if (unknown != null) {
+                if (canceled()) {
+                    return;
+                }
+                for (User u : User.getAll()) { // XXX nice to have a method to iterate these lazily
+                    if (u == unknown) {
+                        continue;
+                    }
+                    if (!users.containsKey(u)) {
+                        synchronized (this) {
+                            users.put(u, new UserInfo(u, null, null));
+                            modified.add(u);
+                        }
+                    }
+                }
+            }
+        }
+
+        @Override protected synchronized JSON data() {
+            JSONArray r = new JSONArray();
+            for (User u : modified) {
+                UserInfo i = users.get(u);
+                JSONObject entry = new JSONObject().
+                        accumulate("id", u.getId()).
+                        accumulate("fullName", u.getFullName()).
+                        accumulate("url", u.getUrl()).
+                        accumulate("avatar", UserAvatarResolver.resolve(u, iconSize)).
+                        accumulate("timeSortKey", i.getTimeSortKey()).
+                        accumulate("lastChangeTimeString", i.getLastChangeTimeString());
+                AbstractProject<?,?> p = i.getProject();
+                if (p != null) {
+                    entry.accumulate("projectUrl", p.getUrl()).accumulate("projectFullDisplayName", p.getFullDisplayName());
+                }
+                r.add(entry);
+            }
+            modified.clear();
+            return r;
+        }
+
+        public Api getApi() {
+            return new Api(parent instanceof Jenkins ? new People((Jenkins) parent) : new People((View) parent));
+        }
+
+    }
+
     void addDisplayNamesToSearchIndex(SearchIndexBuilder sib, Collection<TopLevelItem> items) {
         for(TopLevelItem item : items) {
             
@@ -734,6 +897,7 @@ public abstract class View extends AbstractModelObject implements AccessControll
         rename(req.getParameter("name"));
 
         getProperties().rebuild(req, req.getSubmittedForm(), getApplicablePropertyDescriptors());
+        updateTransientActions();  
 
         save();
 
@@ -806,6 +970,70 @@ public abstract class View extends AbstractModelObject implements AccessControll
             lastBuilds, Run.FEED_ADAPTER_LATEST, req, rsp );
     }
 
+    /**
+     * Accepts <tt>config.xml</tt> submission, as well as serve it.
+     */
+    @WebMethod(name = "config.xml")
+    public HttpResponse doConfigDotXml(StaplerRequest req) throws IOException {
+        if (req.getMethod().equals("GET")) {
+            // read
+            checkPermission(READ);
+            return new HttpResponse() {
+                public void generateResponse(StaplerRequest req, StaplerResponse rsp, Object node) throws IOException, ServletException {
+                    rsp.setContentType("application/xml");
+                    // pity we don't have a handy way to clone Jenkins.XSTREAM to temp add the omit Field
+                    XStream2 xStream2 = new XStream2();
+                    xStream2.omitField(View.class, "owner");
+                    rsp.getOutputStream().write("<?xml version='1.0' encoding='UTF-8'?>\n".getBytes("UTF-8"));
+                    xStream2.toXML(this,  rsp.getOutputStream());
+                }
+            };
+        }
+        if (req.getMethod().equals("POST")) {
+            // submission
+            updateByXml((Source)new StreamSource(req.getReader()));
+            return HttpResponses.ok();
+        }
+
+        // huh?
+        return HttpResponses.error(SC_BAD_REQUEST, "Unexpected request method " + req.getMethod());
+    }
+
+    /**
+     * Updates Job by its XML definition.
+     */
+    public void updateByXml(Source source) throws IOException {
+        checkPermission(CONFIGURE);
+        StringWriter out = new StringWriter();
+        try {
+            // this allows us to use UTF-8 for storing data,
+            // plus it checks any well-formedness issue in the submitted
+            // data
+            Transformer t = TransformerFactory.newInstance()
+                    .newTransformer();
+            t.transform(source,
+                    new StreamResult(out));
+            out.close();
+        } catch (TransformerException e) {
+            throw new IOException2("Failed to persist configuration.xml", e);
+        }
+
+        // try to reflect the changes by reloading
+        InputStream in = new BufferedInputStream(new ByteArrayInputStream(out.toString().getBytes("UTF-8")));
+        try {
+            Jenkins.XSTREAM.unmarshal(new XppDriver().createReader(in), this);
+        } catch (StreamException e) {
+            throw new IOException2("Unable to read",e);
+        } catch(ConversionException e) {
+            throw new IOException2("Unable to read",e);
+        } catch(Error e) {// mostly reflection errors
+            throw new IOException2("Unable to read",e);
+        } finally {
+            in.close();
+        }
+    }
+
+
     /**
      * A list of available view types.
      * @deprecated as of 1.286
@@ -850,14 +1078,28 @@ public abstract class View extends AbstractModelObject implements AccessControll
     
     public static View create(StaplerRequest req, StaplerResponse rsp, ViewGroup owner)
             throws FormException, IOException, ServletException {
+        String requestContentType = req.getContentType();
+        if(requestContentType==null)
+            throw new Failure("No Content-Type header set");
+
+        boolean isXmlSubmission = requestContentType.startsWith("application/xml") || requestContentType.startsWith("text/xml");
+
         String name = req.getParameter("name");
         checkGoodName(name);
         if(owner.getView(name)!=null)
             throw new FormException(Messages.Hudson_ViewAlreadyExists(name),"name");
 
         String mode = req.getParameter("mode");
-        if (mode==null || mode.length()==0)
-            throw new FormException(Messages.View_MissingMode(),"mode");
+        if (mode==null || mode.length()==0) {
+            if(isXmlSubmission) {
+                View v;
+                v = createViewFromXML(name, req.getInputStream());
+                v.owner = owner;
+                rsp.setStatus(HttpServletResponse.SC_OK);
+                return v;
+            } else
+                throw new FormException(Messages.View_MissingMode(),"mode");
+        }
 
         // create a view
         View v = all().findByName(mode).newInstance(req,req.getSubmittedForm());
@@ -869,6 +1111,23 @@ public abstract class View extends AbstractModelObject implements AccessControll
         return v;
     }
 
+    public static View createViewFromXML(String name, InputStream xml) throws IOException {
+        InputStream in = new BufferedInputStream(xml);
+        try {
+            View v = (View) Jenkins.XSTREAM.fromXML(in);
+            v.name = name;
+            return v;
+        } catch(StreamException e) {
+            throw new IOException2("Unable to read",e);
+        } catch(ConversionException e) {
+            throw new IOException2("Unable to read",e);
+        } catch(Error e) {// mostly reflection errors
+            throw new IOException2("Unable to read",e);
+        } finally {
+            in.close();
+        }
+    }
+
     public static class PropertyList extends DescribableList<ViewProperty,ViewPropertyDescriptor> {
         private PropertyList(View owner) {
             super(owner);
