@@ -94,6 +94,22 @@ public class ItemListener implements ExtensionPoint {
     public void onRenamed(Item item, String oldName, String newName) {
     }
 
+    /**
+     * Called after a job has its configuration updated.
+     *
+     * @since 1.460
+     */
+    public void onUpdated(Item item) {
+    }
+
+    /**
+     * @since 1.446
+     *      Called at the begenning of the orderly shutdown sequence to
+     *      allow plugins to clean up stuff
+     */
+    public void onBeforeShutdown() {
+    }
+
     /**
      * Registers this instance to Hudson and start getting notifications.
      *
@@ -120,4 +136,9 @@ public class ItemListener implements ExtensionPoint {
         for (ItemListener l : all())
             l.onCreated(item);
     }
+
+    public static void fireOnUpdated(Item item) {
+        for (ItemListener l : all())
+            l.onUpdated(item);
+    }
 }
