@@ -9,6 +9,7 @@ import org.jruby.ext.posix.POSIX.ERRORS;
 import java.io.File;
 import java.io.InputStream;
 import java.io.PrintStream;
+import java.util.Map;
 import java.util.logging.Logger;
 
 /**
@@ -55,33 +56,35 @@ public class PosixAPI {
         }
 
         public File getCurrentWorkingDirectory() {
-            // TODO
-            throw new UnsupportedOperationException();
+            return new File(".").getAbsoluteFile();
         }
 
         public String[] getEnv() {
-            // TODO
-            throw new UnsupportedOperationException();
+            Map<String,String> envs = System.getenv();
+            String[] envp = new String[envs.size()];
+            
+            int i = 0;
+            for (Map.Entry<String,String> e : envs.entrySet()) {
+                envp[i++] = e.getKey()+'+'+e.getValue();
+            }
+            return envp;
         }
 
         public InputStream getInputStream() {
-            // TODO
-            throw new UnsupportedOperationException();
+            return System.in;
         }
 
         public PrintStream getOutputStream() {
-            // TODO
-            throw new UnsupportedOperationException();
+            return System.out;
         }
 
         public int getPID() {
             // TODO
-            throw new UnsupportedOperationException();
+            return 0;
         }
 
         public PrintStream getErrorStream() {
-            // TODO
-            throw new UnsupportedOperationException();
+            return System.err;
         }
     }, true);
 
