@@ -52,6 +52,7 @@ import org.kohsuke.stapler.QueryParameter;
 import org.kohsuke.stapler.HttpResponse;
 import org.kohsuke.stapler.HttpResponses;
 import org.kohsuke.stapler.HttpRedirect;
+import org.kohsuke.stapler.interceptor.RequirePOST;
 
 import javax.servlet.ServletException;
 import java.io.File;
@@ -69,6 +70,8 @@ import java.util.logging.Logger;
  * @since 1.283
  */
 public class ZFSInstaller extends AdministrativeMonitor implements Serializable {
+    private static final long serialVersionUID = 1018007614648118323L;
+
     /**
      * True if $JENKINS_HOME is a ZFS file system by itself.
      */
@@ -125,8 +128,8 @@ public class ZFSInstaller extends AdministrativeMonitor implements Serializable
     /**
      * Called from the management screen.
      */
+    @RequirePOST
     public HttpResponse doAct(StaplerRequest req) throws ServletException, IOException {
-        requirePOST();
         Jenkins.getInstance().checkPermission(Jenkins.ADMINISTER);
 
         if(req.hasParameter("n")) {
@@ -165,6 +168,8 @@ public class ZFSInstaller extends AdministrativeMonitor implements Serializable
         // this is the actual creation of the file system.
         // return true indicating a success
         return SU.execute(listener, rootUsername, rootPassword, new Callable<String,IOException>() {
+            private static final long serialVersionUID = 7731167233498214301L;
+
             public String call() throws IOException {
                 PrintStream out = listener.getLogger();
 
@@ -211,8 +216,8 @@ public class ZFSInstaller extends AdministrativeMonitor implements Serializable
     /**
      * Called from the confirmation screen to actually initiate the migration.
      */
+    @RequirePOST
     public void doStart(StaplerRequest req, StaplerResponse rsp, @QueryParameter String username, @QueryParameter String password) throws ServletException, IOException {
-        requirePOST(); 
         Jenkins hudson = Jenkins.getInstance();
         hudson.checkPermission(Jenkins.ADMINISTER);
 
