@@ -66,6 +66,9 @@ import java.util.StringTokenizer;
 import java.util.List;
 import java.util.Collections;
 import java.util.Set;
+import java.util.jar.Attributes;
+import java.util.jar.JarFile;
+import java.util.jar.Manifest;
 
 /**
  * Build by using Maven.
@@ -164,6 +167,7 @@ public class Maven extends Builder {
      * name.
      */
     private static final class DecideDefaultMavenCommand implements FileCallable<String> {
+        private static final long serialVersionUID = -2327576423452215146L;
         // command line arguments.
         private final String arguments;
 
@@ -290,8 +294,7 @@ public class Maven extends Builder {
             // The other solution would be to set M2_HOME if we are calling Maven2
             // and MAVEN_HOME for Maven1 (only of use for strange people that
             // are calling Maven2 from Maven1)
-            env.put("M2_HOME",mi.getHome());
-            env.put("MAVEN_HOME",mi.getHome());
+            mi.buildEnvVars(env);
         }
         // just as a precaution
         // see http://maven.apache.org/continuum/faqs.html#how-does-continuum-detect-a-successful-build
@@ -397,6 +400,14 @@ public class Maven extends Builder {
             return new File(getHome());
         }
 
+        @Override
+        public void buildEnvVars(EnvVars env) {
+            String home = getHome();
+            env.put("M2_HOME", home);
+            env.put("MAVEN_HOME", home);
+            env.put("PATH+MAVEN", home + "/bin");
+        }
+
         /**
          * Compares the version of this Maven installation to the minimum required version specified.
          *
@@ -409,12 +420,22 @@ public class Maven extends Builder {
             // FIXME using similar stuff as in the maven plugin could be better 
             // olamy : but will add a dependency on maven in core -> so not so good 
             String mavenVersion = launcher.getChannel().call(new Callable<String,IOException>() {
+                    private static final long serialVersionUID = -4143159957567745621L;
+
                     public String call() throws IOException {
                         File[] jars = new File(getHomeDir(),"lib").listFiles();
                         if(jars!=null) { // be defensive
                             for (File jar : jars) {
-                                if (jar.getName().endsWith("-uber.jar") && jar.getName().startsWith("maven-")) {
-                                    return jar.getName();
+                                if (jar.getName().startsWith("maven-")) {
+                                    JarFile jf = null;
+                                    try {
+                                        jf = new JarFile(jar);
+                                        Manifest manifest = jf.getManifest();
+                                        String version = manifest.getMainAttributes().getValue(Attributes.Name.IMPLEMENTATION_VERSION);
+                                        if(version != null) return version;
+                                    } finally {
+                                        if(jf != null) jf.close();
+                                    }
                                 }
                             }
                         }
@@ -424,15 +445,15 @@ public class Maven extends Builder {
 
             if (!mavenVersion.equals("")) {
                 if (mavenReqVersion == MAVEN_20) {
-                    if(mavenVersion.startsWith("maven-2.") || mavenVersion.startsWith("maven-core-2"))
+                    if(mavenVersion.startsWith("2."))
                         return true;
                 }
                 else if (mavenReqVersion == MAVEN_21) {
-                    if(mavenVersion.startsWith("maven-2.") && !mavenVersion.startsWith("maven-2.0"))
+                    if(mavenVersion.startsWith("2.") && !mavenVersion.startsWith("2.0"))
                         return true;
                 }
                 else if (mavenReqVersion == MAVEN_30) {
-                    if(mavenVersion.startsWith("maven-3.") && !mavenVersion.startsWith("maven-2.0"))
+                    if(mavenVersion.startsWith("3."))
                         return true;
                 }                
             }
@@ -441,7 +462,7 @@ public class Maven extends Builder {
         }
         
         /**
-         * Is this Maven 2.1.x or later?
+         * Is this Maven 2.1.x or 2.2.x - but not Maven 3.x?
          *
          * @param launcher
          *      Represents the node on which we evaluate the path.
@@ -449,28 +470,19 @@ public class Maven extends Builder {
         public boolean isMaven2_1(Launcher launcher) throws IOException, InterruptedException {
             return meetsMavenReqVersion(launcher, MAVEN_21);
         }
-            /*            return launcher.getChannel().call(new Callable<Boolean,IOException>() {
-                public Boolean call() throws IOException {
-                    File[] jars = new File(getHomeDir(),"lib").listFiles();
-                    if(jars!=null) // be defensive
-                        for (File jar : jars)
-                            if(jar.getName().startsWith("maven-2.") && !jar.getName().startsWith("maven-2.0") && jar.getName().endsWith("-uber.jar"))
-                                return true;
-                    return false;
-                }
-                });
-                } */
 
         /**
          * Gets the executable path of this maven on the given target system.
          */
         public String getExecutable(Launcher launcher) throws IOException, InterruptedException {
             return launcher.getChannel().call(new Callable<String,IOException>() {
+                private static final long serialVersionUID = 2373163112639943768L;
+
                 public String call() throws IOException {
-                    File exe = getExeFile("maven");
+                    File exe = getExeFile("mvn");
                     if(exe.exists())
                         return exe.getPath();
-                    exe = getExeFile("mvn");
+                    exe = getExeFile("maven");
                     if(exe.exists())
                         return exe.getPath();
                     return null;
@@ -522,11 +534,15 @@ public class Maven extends Builder {
                 return Collections.singletonList(new MavenInstaller(null));
             }
 
+            // overriding them for backward compatibility.
+            // newer code need not do this
             @Override
             public MavenInstallation[] getInstallations() {
                 return Jenkins.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).getInstallations();
             }
 
+            // overriding them for backward compatibility.
+            // newer code need not do this
             @Override
             public void setInstallations(MavenInstallation... installations) {
                 Jenkins.getInstance().getDescriptorByType(Maven.DescriptorImpl.class).setInstallations(installations);
