@@ -307,7 +307,7 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
      *
      * @see OSProcess#act(ProcessCallable)
      */
-    public static interface ProcessCallable<T> extends Serializable {
+    public interface ProcessCallable<T> extends Serializable {
         /**
          * Performs the computational task on the node where the data is located.
          *
@@ -580,6 +580,8 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
 
         /**
          * Method to destroy a process, given pid.
+         *
+         * Looking at the JavaSE source code, this is using SIGTERM (15)
          */
         private static final Method DESTROY_PROCESS;
 
@@ -1001,7 +1003,7 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
                         }
 
                         void skip0() {
-                            // skip trailing '\0's
+                            // skip padding '\0's
                             while(getByte(offset)=='\0')
                                 offset++;
                         }
@@ -1052,20 +1054,22 @@ public abstract class ProcessTree implements Iterable<OSProcess>, IProcessTree,
                         * \---------------/ 0xffffffff
                         */
 
-                    int nargs = m.readInt();
+                    // I find the Darwin source code of the 'ps' command helpful in understanding how it does this:
+                    // see http://www.opensource.apple.com/source/adv_cmds/adv_cmds-147/ps/print.c
+                    int argc = m.readInt();
                     String args0 = m.readString(); // exec path
+                    m.skip0();
                     try {
-                        for( int i=0; i<nargs; i++) {
-                            m.skip0();
+                        for( int i=0; i<argc; i++) {
                             arguments.add(m.readString());
                         }
                     } catch (IndexOutOfBoundsException e) {
-                        throw new IllegalStateException("Failed to parse arguments: arg0="+args0+", arguments="+arguments+", nargs="+nargs,e);
+                        throw new IllegalStateException("Failed to parse arguments: pid="+pid+", arg0="+args0+", arguments="+arguments+", nargs="+argc+". Please run 'ps e "+pid+"' and report this to https://issues.jenkins-ci.org/browse/JENKINS-9634",e);
                     }
 
-                    // this is how you can read environment variables
+                    // read env vars that follow
                     while(m.peek()!=0)
-                    envVars.addLine(m.readString());
+                        envVars.addLine(m.readString());
                 } catch (IOException e) {
                     // this happens with insufficient permissions, so just ignore the problem.
                 }
