@@ -1,7 +1,7 @@
 /*
  * The MIT License
  * 
- * Copyright (c) 2010, Winston.Prakash@oracle.com
+ * Copyright (c) 2010, Winston.Prakash@oracle.com, CloudBees, Inc.
  * 
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,11 +24,15 @@
 package hudson.views;
 
 import hudson.DescriptorExtensionList;
+import hudson.Extension;
 import hudson.ExtensionPoint;
 import hudson.model.AbstractDescribableImpl;
 import hudson.model.Descriptor;
+import jenkins.model.GlobalConfiguration;
 import jenkins.model.Jenkins;
 import hudson.model.MyViewsProperty;
+import net.sf.json.JSONObject;
+import org.kohsuke.stapler.StaplerRequest;
 
 /**
  * Extension point for adding a MyViewsTabBar header to Projects {@link MyViewsProperty}.
@@ -57,4 +61,30 @@ public abstract class MyViewsTabBar extends AbstractDescribableImpl<MyViewsTabBa
     public MyViewsTabBarDescriptor getDescriptor() {
         return (MyViewsTabBarDescriptor)super.getDescriptor();
     }
+
+    /**
+     * Configures {@link ViewsTabBar} in the system configuration.
+     *
+     * @author Kohsuke Kawaguchi
+     */
+    @Extension(ordinal=305)
+    public static class GlobalConfigurationImpl extends GlobalConfiguration {
+        public MyViewsTabBar getMyViewsTabBar() {
+            return Jenkins.getInstance().getMyViewsTabBar();
+        }
+
+        @Override
+        public boolean configure(StaplerRequest req, JSONObject json) throws FormException {
+            // for compatibility reasons, the actual value is stored in Jenkins
+            Jenkins j = Jenkins.getInstance();
+
+            if (json.has("myViewsTabBar")) {
+                j.setMyViewsTabBar(req.bindJSON(MyViewsTabBar.class,json.getJSONObject("myViewsTabBar")));
+            } else {
+                j.setMyViewsTabBar(new DefaultMyViewsTabBar());
+            }
+
+            return true;
+        }
+    }
 }
