@@ -42,6 +42,7 @@ import java.util.Set;
 import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
+import java.util.concurrent.Future;
 
 import org.apache.commons.io.FileUtils;
 import org.apache.commons.io.output.NullOutputStream;
@@ -66,7 +67,9 @@ public class FilePathTest extends ChannelTestCase {
      * An attempt to reproduce the file descriptor leak.
      * If this operation leaks a file descriptor, 2500 should be enough, I think.
      */
-    public void testCopyTo2() throws Exception {
+    // TODO: this test is much too slow to be a traditional unit test. Should be extracted into some stress test
+    // which is no part of the default test harness?
+    public void testNoFileLeakInCopyTo() throws Exception {
         for (int j=0; j<2500; j++) {
             File tmp = File.createTempFile("testCopyFrom","");
             FilePath f = new FilePath(tmp);
@@ -93,76 +96,89 @@ public class FilePathTest extends ChannelTestCase {
      * Also see JENKINS-7897
      */
     @Bug(7871)
-    public void testCopyTo3() throws Exception {
-        final File tmp = File.createTempFile("testCopyTo3","");
+    public void testNoRaceConditionInCopyTo() throws Exception {
+        final File tmp = File.createTempFile("testNoRaceConditionInCopyTo","");
 
-        FileOutputStream os = new FileOutputStream(tmp);
-        final int size = 90000;
+        try {
+           int fileSize = 90000;
+        
+            givenSomeContentInFile(tmp, fileSize);
+        
+            List<Future<Integer>> results = whenFileIsCopied100TimesConcurrently(tmp);
+
+            // THEN copied count was always equal the expected size
+            for (Future<Integer> f : results)
+                assertEquals(fileSize,f.get().intValue());
+        } finally {
+            tmp.delete();
+        }
+    }
+
+    private void givenSomeContentInFile(File file, int size) throws IOException {
+        FileOutputStream os = new FileOutputStream(file);
         byte[] buf = new byte[size];
         for (int i=0; i<buf.length; i++)
             buf[i] = (byte)(i%256);
         os.write(buf);
         os.close();
+    }
+    
+    private List<Future<Integer>> whenFileIsCopied100TimesConcurrently(final File file) throws InterruptedException {
+        List<Callable<Integer>> r = new ArrayList<Callable<Integer>>();
+        for (int i=0; i<100; i++) {
+            r.add(new Callable<Integer>() {
+                public Integer call() throws Exception {
+                    class Sink extends OutputStream {
+                        private Exception closed;
+                        private volatile int count;
+
+                        private void checkNotClosed() throws IOException2 {
+                            if (closed != null)
+                                throw new IOException2(closed);
+                        }
+
+                        @Override
+                        public void write(int b) throws IOException {
+                            count++;
+                            checkNotClosed();
+                        }
+
+                        @Override
+                        public void write(byte[] b) throws IOException {
+                            count+=b.length;
+                            checkNotClosed();
+                        }
+
+                        @Override
+                        public void write(byte[] b, int off, int len) throws IOException {
+                            count+=len;
+                            checkNotClosed();
+                        }
+
+                        @Override
+                        public void close() throws IOException {
+                            closed = new Exception();
+                            //if (size!=count)
+                            //    fail();
+                        }
+                    }
+
+                    FilePath f = new FilePath(french, file.getPath());
+                    Sink sink = new Sink();
+                    f.copyTo(sink);
+                    return sink.count;
+                }
+            });
+        }
 
         ExecutorService es = Executors.newFixedThreadPool(100);
         try {
-            List<java.util.concurrent.Future<Object>> r = new ArrayList<java.util.concurrent.Future<Object>>();
-            for (int i=0; i<100; i++) {
-                r.add(es.submit(new Callable<Object>() {
-                    public Object call() throws Exception {
-                        class Sink extends OutputStream {
-                            private Exception closed;
-                            private volatile int count;
-
-                            private void checkNotClosed() throws IOException2 {
-                                if (closed != null)
-                                    throw new IOException2(closed);
-                            }
-
-                            @Override
-                            public void write(int b) throws IOException {
-                                count++;
-                                checkNotClosed();
-                            }
-
-                            @Override
-                            public void write(byte[] b) throws IOException {
-                                count+=b.length;
-                                checkNotClosed();
-                            }
-
-                            @Override
-                            public void write(byte[] b, int off, int len) throws IOException {
-                                count+=len;
-                                checkNotClosed();
-                            }
-
-                            @Override
-                            public void close() throws IOException {
-                                closed = new Exception();
-                                if (size!=count)
-                                    fail();
-                            }
-                        }
-
-                        FilePath f = new FilePath(french, tmp.getPath());
-                        Sink sink = new Sink();
-                        f.copyTo(sink);
-                        assertEquals(size,sink.count);
-                        return null;
-                    }
-                }));
-            }
-
-            for (java.util.concurrent.Future<Object> f : r)
-                f.get();
+            return es.invokeAll(r);
         } finally {
             es.shutdown();
         }
     }
 
-
-
     public void testRepeatCopyRecursiveTo() throws Exception {
         // local->local copy used to return 0 if all files were "up to date"
         // should return number of files processed, whether or not they were copied or already current
@@ -408,4 +424,55 @@ public class FilePathTest extends ChannelTestCase {
         assertEquals("/opt/jenkins/workspace/foo/bar/manchu", new FilePath(nixPath, "foo/bar\\manchu").getRemote());
         assertEquals("/opt/jenkins/workspace/foo/bar/manchu", new FilePath(nixPath, "foo/bar/manchu").getRemote());
     }
+
+    public void testValidateAntFileMask() throws Exception {
+        File tmp = Util.createTempDir();
+        try {
+            FilePath d = new FilePath(french, tmp.getPath());
+            d.child("d1/d2/d3").mkdirs();
+            d.child("d1/d2/d3/f.txt").touch(0);
+            d.child("d1/d2/d3/f.html").touch(0);
+            d.child("d1/d2/f.txt").touch(0);
+            assertValidateAntFileMask(null, d, "**/*.txt");
+            assertValidateAntFileMask(null, d, "d1/d2/d3/f.txt");
+            assertValidateAntFileMask(null, d, "**/*.html");
+            assertValidateAntFileMask(Messages.FilePath_validateAntFileMask_portionMatchButPreviousNotMatchAndSuggest("**/*.js", "**", "**/*.js"), d, "**/*.js");
+            assertValidateAntFileMask(Messages.FilePath_validateAntFileMask_doesntMatchAnything("index.htm"), d, "index.htm");
+            assertValidateAntFileMask(Messages.FilePath_validateAntFileMask_doesntMatchAndSuggest("f.html", "d1/d2/d3/f.html"), d, "f.html");
+            // XXX lots more to test, e.g. multiple patterns separated by commas; ought to have full code coverage for this method
+        } finally {
+            Util.deleteRecursive(tmp);
+        }
+    }
+
+    private static void assertValidateAntFileMask(String expected, FilePath d, String fileMasks) throws Exception {
+        assertEquals(expected, d.validateAntFileMask(fileMasks));
+    }
+
+    @Bug(7214)
+    public void testValidateAntFileMaskBounded() throws Exception {
+        File tmp = Util.createTempDir();
+        try {
+            FilePath d = new FilePath(french, tmp.getPath());
+            FilePath d2 = d.child("d1/d2");
+            d2.mkdirs();
+            for (int i = 0; i < 100; i++) {
+                FilePath d3 = d2.child("d" + i);
+                d3.mkdirs();
+                d3.child("f.txt").touch(0);
+            }
+            assertEquals(null, d.validateAntFileMask("d1/d2/**/f.txt"));
+            assertEquals(null, d.validateAntFileMask("d1/d2/**/f.txt", 10));
+            assertEquals(Messages.FilePath_validateAntFileMask_portionMatchButPreviousNotMatchAndSuggest("**/*.js", "**", "**/*.js"), d.validateAntFileMask("**/*.js", 1000));
+            try {
+                d.validateAntFileMask("**/*.js", 10);
+                fail();
+            } catch (InterruptedException x) {
+                // good
+            }
+        } finally {
+            Util.deleteRecursive(tmp);
+        }
+    }
+
 }
