@@ -30,6 +30,8 @@ import hudson.maven.MavenBuildProxy;
 import hudson.model.Api;
 import hudson.model.BuildListener;
 import hudson.model.FingerprintMap;
+import hudson.model.Run;
+import hudson.util.LRUStringConverter;
 import jenkins.model.Jenkins;
 
 import hudson.util.HttpResponses;
@@ -68,6 +70,11 @@ import java.util.logging.Logger;
  */
 @ExportedBean
 public final class MavenArtifact implements Serializable {
+
+    static {
+        Run.XSTREAM.registerLocalConverter(MavenArtifact.class, "md5sum", new LRUStringConverter(5000));
+    }
+
     /**
      * Basic parameters of a Maven artifact.
      */
@@ -141,13 +148,13 @@ public final class MavenArtifact implements Serializable {
         File file = a.getFile();
         if(file==null)
             return null; // perhaps build failed and didn't leave an artifact
-        if(!file.exists() || file.isDirectory())
-            return null; // during a build maven sets a class folder instead of a jar file as artifact. ignore.
+        if(!file.isFile())
+            return null; // file doesn't exist or artifact points to a directory
         return new MavenArtifact(a);
     }
 
     public boolean isPOM() {
-        return fileName.endsWith(".pom");   // hack
+        return fileName.endsWith(".pom")||"pom.xml".equals(fileName);   // hack
     }
 
     /**
@@ -180,7 +187,7 @@ public final class MavenArtifact implements Serializable {
      */
     private String getSeed(String extension) {
         String name = artifactId+'-'+version;
-        if(classifier!=null)
+        if(Util.fixEmpty(classifier)!=null)
             name += '-'+classifier;
         name += '.'+extension;
         return name;
