@@ -2,6 +2,9 @@ package jenkins.plugins.ui_samples;
 
 import hudson.Extension;
 import hudson.model.RootAction;
+import jenkins.model.ModelObjectWithContextMenu;
+import org.kohsuke.stapler.StaplerRequest;
+import org.kohsuke.stapler.StaplerResponse;
 
 import java.util.List;
 
@@ -11,9 +14,9 @@ import java.util.List;
  * @author Kohsuke Kawaguchi
  */
 @Extension
-public class Root implements RootAction {
+public class Root implements RootAction, ModelObjectWithContextMenu {
     public String getIconFileName() {
-        return "gear.gif";
+        return "gear.png";
     }
 
     public String getDisplayName() {
@@ -34,4 +37,8 @@ public class Root implements RootAction {
     public List<UISample> getAll() {
         return UISample.all();
     }
+
+    public ContextMenu doContextMenu(StaplerRequest request, StaplerResponse response) throws Exception {
+        return new ContextMenu().addAll(getAll());
+    }
 }
