@@ -39,5 +39,11 @@ do
       # composite -compose Dst_Over -tile xc:white t.png $dst
       # rm t.png
     fi
+
+    gif=$(echo $dst | sed -e s/.png/.gif/)
+    if [ ! -e $gif -o $dst -nt $gif ];
+    then
+      convert $dst -background white -flatten -transparent white $gif
+    fi
   done
 done
